package players {
	
	import flash.events.TimerEvent;
	import flash.utils.Timer;
	
	import mx.graphics.IFill;
	import mx.graphics.SolidColor;

	public class EnemyPlayerModel extends PlayerModel
	{
				
		//The player - we need to detect if we hit this object
		private var player:PlayerModel;
		
		//We will update our position whenever this timer goes off - every 3ms
		private var positionTimer:Timer = new Timer(3);
		
		//Keep track of the width and height of the stage so we know when to bounce off the wall
		private var stageWidth:int;
		private var stageHeight:int;
		
		//Function that gets called when we hit something
		private var onHit:Function;
		
		public function EnemyPlayerModel(_x:int, _y:int, _radius:int, _fill:IFill, _xVelocity:int, _yVelocity:int, _player:PlayerModel, width:int, height:int, _onHit:Function)
		{
			super(_x, _y, _radius, _fill, _xVelocity, _yVelocity);
			
			this.player = _player;
			
			this.positionTimer.addEventListener(TimerEvent.TIMER, this.timerUpdate);
			this.positionTimer.start();
			
			this.stageWidth = width;
			this.stageHeight = height;
			
			this.onHit = _onHit;
		}
		
		//Whenever the timer goes off, update our position, check if we hit the player
		private function timerUpdate(event:TimerEvent):void {
			this.updatePosition();
			if(this.hitPlayer()) {
				this.onHit(this.radius);
			}
		}
		
		private function updatePosition():void {
			this.x += this.xVelocity;
			this.y += this.yVelocity;
			
			//Bounce off the left wall?
			if(this.x+this.radius>=stageWidth) {
				this.x = stageWidth - this.radius;
				this.xVelocity = this.xVelocity * -1;
			}
			//Bounce off the right wall?
			else if(this.x-this.radius<=0) {
				this.x = this.radius;
				this.xVelocity = this.xVelocity * -1;
			}
			
			//Boucne off the bottom wall?
			if(this.y+this.radius>=stageHeight) {
				this.y = stageHeight - this.radius;
				this.yVelocity = this.yVelocity * -1;
			}
			//Bounce off the top wall?
			else if(this.y-this.radius<=0) {
				this.y = this.radius;
				this.yVelocity = this.yVelocity * -1;
			}
		}
		
		//Did we hit the player?
		private function hitPlayer():Boolean {
			var xDistance:int = this.x - this.player.x;
			var yDistance:int = this.y - this.player.y;
			
			//By Pythagorean theorem. Math.pow takes the power of something - Math.pow(x,2) is x squared
			if(Math.pow(xDistance, 2) + Math.pow(yDistance, 2) < Math.pow(this.radius + this.player.radius, 2))
				return true;
			else
				return false;
		}
		
		//If the user resizes the window, we want to know our updated dimensions
		public function updateWidthAndHeight(newWidth:int, newHeight:int):void {
			this.stageWidth = newWidth;
			this.stageHeight = newHeight;
		}
	}
}